@@ -1,6 +1,7 @@
 import asyncio
 import functools
 import multiprocessing
+import sys
 import threading
 import time
 from concurrent.futures import ThreadPoolExecutor
@@ -9,6 +10,7 @@ from unittest import TestCase
 
 import pytest
 
+from asgiref.compatibility import create_task, get_running_loop
 from asgiref.sync import ThreadSensitiveContext, async_to_sync, sync_to_async
 
 
@@ -33,12 +35,17 @@ async def test_sync_to_async():
     assert result == 42
     assert end - start >= 1
     # Set workers to 1, call it twice and make sure that works right
-    loop = asyncio.get_event_loop()
-    old_executor = loop._default_executor
+    loop = get_running_loop()
+    old_executor = loop._default_executor or ThreadPoolExecutor()
     loop.set_default_executor(ThreadPoolExecutor(max_workers=1))
     try:
         start = time.monotonic()
-        await asyncio.wait([async_function(), async_function()])
+        await asyncio.wait(
+            [
+                create_task(async_function()),
+                create_task(async_function()),
+            ]
+        )
         end = time.monotonic()
         # It should take at least 2 seconds as there's only one worker.
         assert end - start >= 2
@@ -428,7 +435,7 @@ async def test_thread_sensitive_outside_async():
         result["thread"] = threading.current_thread()
 
     # Run it (in supposed parallel!)
-    await asyncio.wait([outer(result_1), inner(result_2)])
+    await asyncio.wait([create_task(outer(result_1)), create_task(inner(result_2))])
 
     # They should not have run in the main thread, but in the same thread
     assert result_1["thread"] != threading.current_thread()
@@ -449,7 +456,10 @@ async def test_thread_sensitive_with_context_matches():
         async with ThreadSensitiveContext():
             # Run it (in supposed parallel!)
             await asyncio.wait(
-                [store_thread_async(result_1), store_thread_async(result_2)]
+                [
+                    create_task(store_thread_async(result_1)),
+                    create_task(store_thread_async(result_2)),
+                ]
             )
 
     await fn()
@@ -660,3 +670,50 @@ async def test_sync_to_async_uses_executor():
         thread_sensitive=True,
         executor=custom_executor,
     )
+
+
+@pytest.mark.skipif(sys.version_info < (3, 7), reason="Issue persists with 3.6")
+def test_sync_to_async_deadlock_raises():
+    def db_write():
+        pass
+
+    async def io_task():
+        await sync_to_async(db_write)()
+
+    async def do_io_tasks():
+        t = asyncio.create_task(io_task())
+        await t
+        # await asyncio.gather(io_task()) # Also deadlocks
+        # await io_task() # Works
+
+    def view():
+        async_to_sync(do_io_tasks)()
+
+    async def server_entry():
+        await sync_to_async(view)()
+
+    with pytest.raises(RuntimeError):
+        asyncio.run(server_entry())
+
+
+@pytest.mark.skipif(sys.version_info < (3, 7), reason="Issue persists with 3.6")
+def test_sync_to_async_deadlock_ignored_with_exception():
+    """
+    Ensures that throwing an exception from inside a deadlock-protected block
+    still resets the deadlock detector's status.
+    """
+
+    def view():
+        raise ValueError()
+
+    async def server_entry():
+        try:
+            await sync_to_async(view)()
+        except ValueError:
+            pass
+        try:
+            await sync_to_async(view)()
+        except ValueError:
+            pass
+
+    asyncio.run(server_entry())
